@@ -22,33 +22,6 @@ int CPUMeter_attributes[] = {
    CPU_NICE, CPU_NORMAL, CPU_KERNEL, CPU_IRQ, CPU_SOFTIRQ, CPU_IOWAIT
 };
 
-MeterType CPUMeter = {
-   .setValues = CPUMeter_setValues, 
-   .display = CPUMeter_display,
-   .mode = BAR_METERMODE,
-   .items = 6,
-   .total = 100.0,
-   .attributes = CPUMeter_attributes, 
-   .name = "CPU",
-   .uiName = "CPU",
-   .caption = "CPU",
-   .init = CPUMeter_init
-};
-
-MeterType AllCPUsMeter = {
-   .mode = 0,
-   .items = 1,
-   .total = 100.0,
-   .attributes = CPUMeter_attributes, 
-   .name = "AllCPUs",
-   .uiName = "All CPUs",
-   .caption = "CPU",
-   .draw = AllCPUsMeter_draw,
-   .init = AllCPUsMeter_init,
-   .setMode = AllCPUsMeter_setMode,
-   .done = AllCPUsMeter_done
-};
-
 #ifndef MIN
 #define MIN(a,b) ((a)<(b)?(a):(b))
 #endif
@@ -56,7 +29,7 @@ MeterType AllCPUsMeter = {
 #define MAX(a,b) ((a)>(b)?(a):(b))
 #endif
 
-void CPUMeter_init(Meter* this) {
+static void CPUMeter_init(Meter* this) {
    int processor = this->param;
    if (this->pl->processorCount > 1) {
       char caption[10];
@@ -67,7 +40,7 @@ void CPUMeter_init(Meter* this) {
       Meter_setCaption(this, "Avg");
 }
 
-void CPUMeter_setValues(Meter* this, char* buffer, int size) {
+static void CPUMeter_setValues(Meter* this, char* buffer, int size) {
    ProcessList* pl = this->pl;
    int processor = this->param;
    double total = (double) pl->totalPeriod[processor];
@@ -90,7 +63,7 @@ void CPUMeter_setValues(Meter* this, char* buffer, int size) {
    snprintf(buffer, size, "%5.1f%%", cpu );
 }
 
-void CPUMeter_display(Object* cast, RichString* out) {
+static void CPUMeter_display(Object* cast, RichString* out) {
    char buffer[50];
    Meter* this = (Meter*)cast;
    RichString_init(out);
@@ -123,7 +96,7 @@ void CPUMeter_display(Object* cast, RichString* out) {
    }
 }
 
-void AllCPUsMeter_init(Meter* this) {
+static void AllCPUsMeter_init(Meter* this) {
    int processors = this->pl->processorCount;
    this->drawBuffer = malloc(sizeof(Meter*) * processors);
    Meter** meters = (Meter**) this->drawBuffer;
@@ -133,21 +106,21 @@ void AllCPUsMeter_init(Meter* this) {
    this->mode = BAR_METERMODE;
 }
 
-void AllCPUsMeter_done(Meter* this) {
+static void AllCPUsMeter_done(Meter* this) {
    int processors = this->pl->processorCount;
    Meter** meters = (Meter**) this->drawBuffer;
    for (int i = 0; i < processors; i++)
       Meter_delete((Object*)meters[i]);
 }
 
-void AllCPUsMeter_setMode(Meter* this, int mode) {
+static void AllCPUsMeter_setMode(Meter* this, int mode) {
    this->mode = mode;
    int processors = this->pl->processorCount;
    int h = Meter_modes[this->mode]->h;
    this->h = h * processors;
 }
 
-void AllCPUsMeter_draw(Meter* this, int x, int y, int w) {
+static void AllCPUsMeter_draw(Meter* this, int x, int y, int w) {
    int processors = this->pl->processorCount;
    Meter** meters = (Meter**) this->drawBuffer;
    for (int i = 0; i < processors; i++) {
@@ -156,3 +129,30 @@ void AllCPUsMeter_draw(Meter* this, int x, int y, int w) {
       y += meters[i]->h;
    }
 }
+
+MeterType CPUMeter = {
+   .setValues = CPUMeter_setValues, 
+   .display = CPUMeter_display,
+   .mode = BAR_METERMODE,
+   .items = 6,
+   .total = 100.0,
+   .attributes = CPUMeter_attributes, 
+   .name = "CPU",
+   .uiName = "CPU",
+   .caption = "CPU",
+   .init = CPUMeter_init
+};
+
+MeterType AllCPUsMeter = {
+   .mode = 0,
+   .items = 1,
+   .total = 100.0,
+   .attributes = CPUMeter_attributes, 
+   .name = "AllCPUs",
+   .uiName = "All CPUs",
+   .caption = "CPU",
+   .draw = AllCPUsMeter_draw,
+   .init = AllCPUsMeter_init,
+   .setMode = AllCPUsMeter_setMode,
+   .done = AllCPUsMeter_done
+};
